/* fd-rr-config.h
 * -*- c-basic-offset: 4 -*-
 *
 * Copyright 2007, 2008, Red Hat, Inc.
 * Copyright 2010 Giovanni Campagna
 *
 * This file is part of the Mate Library.
 *
 * The Mate Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Mate Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the Mate Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Author: Soren Sandmann <sandmann@redhat.com>
 */
#ifndef FD_RR_CONFIG_H
#define FD_RR_CONFIG_H

// #ifndef MATE_DESKTOP_USE_UNSTABLE_API
// #error   fd-rr-config.h is unstable API. You must define MATE_DESKTOP_USE_UNSTABLE_API before including fd-rr-config.h
// #endif

#include "fd-rr.h"
#include <glib.h>
#include <glib-object.h>

typedef struct FdRROutputInfoPrivate FdRROutputInfoPrivate;
typedef struct FdRRConfigPrivate FdRRConfigPrivate;

typedef struct
{
    GObject parent;

    /*< private >*/
    FdRROutputInfoPrivate *priv;
} FdRROutputInfo;

typedef struct
{
    GObjectClass parent_class;
} FdRROutputInfoClass;

#define FD_TYPE_RR_OUTPUT_INFO                  (fd_rr_output_info_get_type())
#define FD_RR_OUTPUT_INFO(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), FD_TYPE_RR_OUTPUT_INFO, FdRROutputInfo))
#define FD_IS_RR_OUTPUT_INFO(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FD_TYPE_RR_OUTPUT_INFO))
#define FD_RR_OUTPUT_INFO_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), FD_TYPE_RR_OUTPUT_INFO, FdRROutputInfoClass))
#define FD_IS_RR_OUTPUT_INFO_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), FD_TYPE_RR_OUTPUT_INFO))
#define FD_RR_OUTPUT_INFO_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), FD_TYPE_RR_OUTPUT_INFO, FdRROutputInfoClass))

GType fd_rr_output_info_get_type (void);

char *fd_rr_output_info_get_name (FdRROutputInfo *self);

gboolean fd_rr_output_info_is_active  (FdRROutputInfo *self);
void     fd_rr_output_info_set_active (FdRROutputInfo *self, gboolean active);


void fd_rr_output_info_get_geometry (FdRROutputInfo *self, int *x, int *y, int *width, int *height);
void fd_rr_output_info_set_geometry (FdRROutputInfo *self, int  x, int  y, int  width, int  height);

int  fd_rr_output_info_get_refresh_rate (FdRROutputInfo *self);
void fd_rr_output_info_set_refresh_rate (FdRROutputInfo *self, int rate);

FdRRRotation fd_rr_output_info_get_rotation (FdRROutputInfo *self);
void            fd_rr_output_info_set_rotation (FdRROutputInfo *self, FdRRRotation rotation);

gboolean fd_rr_output_info_is_connected     (FdRROutputInfo *self);
void     fd_rr_output_info_get_vendor       (FdRROutputInfo *self, gchar* vendor);
guint    fd_rr_output_info_get_product      (FdRROutputInfo *self);
guint    fd_rr_output_info_get_serial       (FdRROutputInfo *self);
double   fd_rr_output_info_get_aspect_ratio (FdRROutputInfo *self);
char    *fd_rr_output_info_get_display_name (FdRROutputInfo *self);

gboolean fd_rr_output_info_get_primary (FdRROutputInfo *self);
void     fd_rr_output_info_set_primary (FdRROutputInfo *self, gboolean primary);

int fd_rr_output_info_get_preferred_width  (FdRROutputInfo *self);
int fd_rr_output_info_get_preferred_height (FdRROutputInfo *self);

typedef struct
{
    GObject parent;

    /*< private >*/
    FdRRConfigPrivate *priv;
} FdRRConfig;

typedef struct
{
    GObjectClass parent_class;
} FdRRConfigClass;

#define FD_TYPE_RR_CONFIG                  (fd_rr_config_get_type())
#define FD_RR_CONFIG(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), FD_TYPE_RR_CONFIG, FdRRConfig))
#define FD_IS_RR_CONFIG(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FD_TYPE_RR_CONFIG))
#define FD_RR_CONFIG_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), FD_TYPE_RR_CONFIG, FdRRConfigClass))
#define FD_IS_RR_CONFIG_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), FD_TYPE_RR_CONFIG))
#define FD_RR_CONFIG_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), FD_TYPE_RR_CONFIG, FdRRConfigClass))

GType               fd_rr_config_get_type     (void);

FdRRConfig      *fd_rr_config_new_current  (FdRRScreen  *screen,
						  GError        **error);
FdRRConfig      *fd_rr_config_new_stored   (FdRRScreen  *screen,
						  GError        **error);
gboolean                fd_rr_config_load_current (FdRRConfig  *self,
						      GError        **error);
gboolean                fd_rr_config_load_filename (FdRRConfig  *self,
						       const gchar    *filename,
						       GError        **error);
gboolean            fd_rr_config_match        (FdRRConfig  *config1,
						  FdRRConfig  *config2);
gboolean            fd_rr_config_equal	 (FdRRConfig  *config1,
						  FdRRConfig  *config2);
gboolean            fd_rr_config_save         (FdRRConfig  *configuration,
						  GError        **error);
void                fd_rr_config_sanitize     (FdRRConfig  *configuration);
gboolean            fd_rr_config_ensure_primary (FdRRConfig  *configuration);

gboolean	    fd_rr_config_apply_with_time (FdRRConfig  *configuration,
						     FdRRScreen  *screen,
						     guint32         timestamp,
						     GError        **error);

gboolean            fd_rr_config_apply_from_filename_with_time (FdRRScreen  *screen,
								   const char     *filename,
								   guint32         timestamp,
								   GError        **error);

gboolean            fd_rr_config_applicable   (FdRRConfig  *configuration,
						  FdRRScreen  *screen,
						  GError        **error);

gboolean            fd_rr_config_get_clone    (FdRRConfig  *configuration);
void                fd_rr_config_set_clone    (FdRRConfig  *configuration, gboolean clone);
FdRROutputInfo **fd_rr_config_get_outputs  (FdRRConfig  *configuration);

char *fd_rr_config_get_backup_filename (void);
char *fd_rr_config_get_intended_filename (void);

#endif
