/* fd-rr.h
 *
 * Copyright 2007, 2008, Red Hat, Inc.
 *
 * This file is part of the Mate Library.
 *
 * The Mate Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Mate Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with the Mate Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Author: Soren Sandmann <sandmann@redhat.com>
 */
#ifndef FD_RR_H
#define FD_RR_H

// #ifndef MATE_DESKTOP_USE_UNSTABLE_API
// #error    FdRR is unstable API. You must define MATE_DESKTOP_USE_UNSTABLE_API before including fdrr.h
// #endif

#include <glib.h>
#include <gdk/gdk.h>

typedef struct FdRRScreenPrivate FdRRScreenPrivate;
typedef struct FdRROutput FdRROutput;
typedef struct FdRRCrtc FdRRCrtc;
typedef struct FdRRMode FdRRMode;

typedef struct {
    GObject parent;

    FdRRScreenPrivate* priv;
} FdRRScreen;

typedef struct {
    GObjectClass parent_class;

        void (* changed) (void);
} FdRRScreenClass;

typedef enum
{
    FD_RR_ROTATION_0 =	(1 << 0),
    FD_RR_ROTATION_90 =	(1 << 1),
    FD_RR_ROTATION_180 =	(1 << 2),
    FD_RR_ROTATION_270 =	(1 << 3),
    FD_RR_REFLECT_X =	(1 << 4),
    FD_RR_REFLECT_Y =	(1 << 5)
} FdRRRotation;

/* Error codes */

#define FD_RR_ERROR (fd_rr_error_quark ())

GQuark fd_rr_error_quark (void);

typedef enum {
    FD_RR_ERROR_UNKNOWN,		/* generic "fail" */
    FD_RR_ERROR_NO_RANDR_EXTENSION,	/* RANDR extension is not present */
    FD_RR_ERROR_RANDR_ERROR,		/* generic/undescribed error from the underlying XRR API */
    FD_RR_ERROR_BOUNDS_ERROR,	/* requested bounds of a CRTC are outside the maximum size */
    FD_RR_ERROR_CRTC_ASSIGNMENT,	/* could not assign CRTCs to outputs */
    FD_RR_ERROR_NO_MATCHING_CONFIG,	/* none of the saved configurations matched the current configuration */
} FdRRError;

#define FD_RR_CONNECTOR_TYPE_PANEL "Panel"  /* This is a laptop's built-in LCD */

#define FD_TYPE_RR_SCREEN                  (fd_rr_screen_get_type())
#define FD_RR_SCREEN(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), FD_TYPE_RR_SCREEN, FdRRScreen))
#define FD_IS_RR_SCREEN(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FD_TYPE_RR_SCREEN))
#define FD_RR_SCREEN_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), FD_TYPE_RR_SCREEN, FdRRScreenClass))
#define FD_IS_RR_SCREEN_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), FD_TYPE_RR_SCREEN))
#define FD_RR_SCREEN_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), FD_TYPE_RR_SCREEN, FdRRScreenClass))

#define FD_TYPE_RR_OUTPUT (fd_rr_output_get_type())
#define FD_TYPE_RR_CRTC   (fd_rr_crtc_get_type())
#define FD_TYPE_RR_MODE   (fd_rr_mode_get_type())

GType fd_rr_screen_get_type (void);
GType fd_rr_output_get_type (void);
GType fd_rr_crtc_get_type (void);
GType fd_rr_mode_get_type (void);

/* FdRRScreen */
FdRRScreen * fd_rr_screen_new                (GdkScreen             *screen,
						    GError               **error);
FdRROutput **fd_rr_screen_list_outputs       (FdRRScreen         *screen);
FdRRCrtc **  fd_rr_screen_list_crtcs         (FdRRScreen         *screen);
FdRRMode **  fd_rr_screen_list_modes         (FdRRScreen         *screen);
FdRRMode **  fd_rr_screen_list_clone_modes   (FdRRScreen	  *screen);
void            fd_rr_screen_set_size           (FdRRScreen         *screen,
						    int                    width,
						    int                    height,
						    int                    mm_width,
						    int                    mm_height);
FdRRCrtc *   fd_rr_screen_get_crtc_by_id     (FdRRScreen         *screen,
						    guint32                id);
gboolean        fd_rr_screen_refresh            (FdRRScreen         *screen,
						    GError               **error);
FdRROutput * fd_rr_screen_get_output_by_id   (FdRRScreen         *screen,
						    guint32                id);
FdRROutput * fd_rr_screen_get_output_by_name (FdRRScreen         *screen,
						    const char            *name);
void            fd_rr_screen_get_ranges         (FdRRScreen         *screen,
						    int                   *min_width,
						    int                   *max_width,
						    int                   *min_height,
						    int                   *max_height);
void            fd_rr_screen_get_timestamps     (FdRRScreen         *screen,
						    guint32               *change_timestamp_ret,
						    guint32               *config_timestamp_ret);

void            fd_rr_screen_set_primary_output (FdRRScreen         *screen,
                                                    FdRROutput         *output);

/* FdRROutput */
guint32         fd_rr_output_get_id             (FdRROutput         *output);
const char *    fd_rr_output_get_name           (FdRROutput         *output);
gboolean        fd_rr_output_is_connected       (FdRROutput         *output);
int             fd_rr_output_get_size_inches    (FdRROutput         *output);
int             fd_rr_output_get_width_mm       (FdRROutput         *outout);
int             fd_rr_output_get_height_mm      (FdRROutput         *output);
const guint8 *  fd_rr_output_get_edid_data      (FdRROutput         *output);
FdRRCrtc **  fd_rr_output_get_possible_crtcs (FdRROutput         *output);
FdRRMode *   fd_rr_output_get_current_mode   (FdRROutput         *output);
FdRRCrtc *   fd_rr_output_get_crtc           (FdRROutput         *output);
const char *    fd_rr_output_get_connector_type (FdRROutput         *output);
gboolean        fd_rr_output_is_laptop          (FdRROutput         *output);
void            fd_rr_output_get_position       (FdRROutput         *output,
						    int                   *x,
						    int                   *y);
gboolean        fd_rr_output_can_clone          (FdRROutput         *output,
						    FdRROutput         *clone);
FdRRMode **  fd_rr_output_list_modes         (FdRROutput         *output);
FdRRMode *   fd_rr_output_get_preferred_mode (FdRROutput         *output);
gboolean        fd_rr_output_supports_mode      (FdRROutput         *output,
						    FdRRMode           *mode);
gboolean        fd_rr_output_get_is_primary     (FdRROutput         *output);

/* FdRRMode */
guint32         fd_rr_mode_get_id               (FdRRMode           *mode);
guint           fd_rr_mode_get_width            (FdRRMode           *mode);
guint           fd_rr_mode_get_height           (FdRRMode           *mode);
int             fd_rr_mode_get_freq             (FdRRMode           *mode);

/* FdRRCrtc */
guint32         fd_rr_crtc_get_id               (FdRRCrtc           *crtc);

// #ifndef MATE_DISABLE_DEPRECATED
gboolean        fd_rr_crtc_set_config           (FdRRCrtc           *crtc,
						    int                    x,
						    int                    y,
						    FdRRMode           *mode,
						    FdRRRotation        rotation,
						    FdRROutput        **outputs,
						    int                    n_outputs,
						    GError               **error);
// #endif

gboolean        fd_rr_crtc_set_config_with_time (FdRRCrtc           *crtc,
						    guint32                timestamp,
						    int                    x,
						    int                    y,
						    FdRRMode           *mode,
						    FdRRRotation        rotation,
						    FdRROutput        **outputs,
						    int                    n_outputs,
						    GError               **error);
gboolean        fd_rr_crtc_can_drive_output     (FdRRCrtc           *crtc,
						    FdRROutput         *output);
FdRRMode *   fd_rr_crtc_get_current_mode     (FdRRCrtc           *crtc);
void            fd_rr_crtc_get_position         (FdRRCrtc           *crtc,
						    int                   *x,
						    int                   *y);
FdRRRotation fd_rr_crtc_get_current_rotation (FdRRCrtc           *crtc);
FdRRRotation fd_rr_crtc_get_rotations        (FdRRCrtc           *crtc);
gboolean        fd_rr_crtc_supports_rotation    (FdRRCrtc           *crtc,
						    FdRRRotation        rotation);

gboolean        fd_rr_crtc_get_gamma            (FdRRCrtc           *crtc,
						    int                   *size,
						    unsigned short       **red,
						    unsigned short       **green,
						    unsigned short       **blue);
void            fd_rr_crtc_set_gamma            (FdRRCrtc           *crtc,
						    int                    size,
						    unsigned short        *red,
						    unsigned short        *green,
						    unsigned short        *blue);
#endif /* FD_RR_H */
